// High level document window stuff
// Copyright (C) 2000 Core Technologies.

// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".

#include	"includes.h"

EDITOR_WINDOW *LocateEditorDocumentWindow(char *bufferName)
// attempt to locate a preexisting document window that
// is open to absolutePath, if one is found, return it
// if none is located, return NULL
{
	EDITOR_BUFFER
		*buffer;

	if((buffer=LocateBuffer(bufferName)))
	{
		return(buffer->window);
	}
	return(NULL);
}

void EditorReassignDocumentWindowTitle(EDITOR_BUFFER *buffer)
// if buffer has a window associated with it, assign the
// window's title based on buffer's contentName
// if there is no window, do nothing
{
	char
		*windowTitle;

	if(buffer->window)
	{
		if(buffer->fromFile)
		{
			if((windowTitle=CreateWindowTitleFromPath(buffer->contentName)))
			{
				SetDocumentWindowTitle(buffer->window,windowTitle);			// change the window's title
				MDisposePtr(windowTitle);									// get rid of memory
			}
			else
			{
				SetDocumentWindowTitle(buffer->window,buffer->contentName);	// had problem getting new name, use path
			}
		}
		else
		{
			SetDocumentWindowTitle(buffer->window,buffer->contentName);		// change the window's title
		}
	}
}

EDITOR_WINDOW *EditorOpenDocumentWindow(EDITOR_BUFFER *buffer,EDITOR_RECT *rect,char *fontName,UINT32 tabSize,EDITOR_COLOR foreground,EDITOR_COLOR background)
// create a new document edit window for buffer, place it as the front-most
// NOTE: if there is already a document window for buffer
// place it front-most, and return it
// if there is a problem, SetError, return NULL
{
	EDITOR_WINDOW
		*window;
	EDITOR_VIEW
		*view;
	char
		*titleToDelete,
		*windowTitle;

	if(buffer->window)
	{
		view=GetDocumentWindowCurrentView(buffer->window);
		SetEditorDocumentWindowRect(buffer->window,rect);			// adjust the rectangle
		SetEditorViewStyleFont(view,0,fontName);					// and the font
		SetEditorViewTabSize(view,tabSize);							// and the tabsize
		SetEditorViewStyleForegroundColor(view,0,foreground);		// set colors for style 0
		SetEditorViewStyleBackgroundColor(view,0,background);
		SetEditorDocumentWindowForegroundColor(buffer->window,foreground);	// set the new colors
		SetEditorDocumentWindowBackgroundColor(buffer->window,background);
		SetTopDocumentWindow(buffer->window);						// place it on top
	}
	else
	{
		titleToDelete=NULL;
		if(buffer->fromFile)
		{
			titleToDelete=CreateWindowTitleFromPath(buffer->contentName);
		}
		if(titleToDelete)
		{
			windowTitle=titleToDelete;
		}
		else
		{
			windowTitle=buffer->contentName;
		}
		if((window=OpenDocumentWindow(buffer,rect,windowTitle,fontName,tabSize,foreground,background)))
		{
			buffer->window=window;
		}
		if(titleToDelete)
		{
			MDisposePtr(titleToDelete);
		}
	}
	return(buffer->window);
}

void EditorCloseDocumentWindow(EDITOR_WINDOW *window)
// close the given edit window (do not destroy the buffer it is tied to)
// this is not allowed to fail
{
	window->buffer->window=NULL;							// point the buffer that had this window to NULL
	CloseDocumentWindow(window);							// get rid of the window
}
