// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".


// view tracking modes

#define	TM_mMODE			0x000F		// masks out selection modes
#define	TM_CHAR				0x0000		// select by character
#define	TM_WORD				0x0001		// select by word
#define	TM_LINE				0x0002		// select by line
#define	TM_ALL				0x0003		// select everything
#define	TM_mHAND			0x1000		// tells if the hand movement tool has been selected
#define	TM_mCOLUMNAR		0x2000		// tells if the selection should be columnar
#define	TM_mREPEAT			0x4000		// tells if still tracking the same selection
#define	TM_mCONTINUE		0x8000		// tells if the selection should be continued from what is currently selected

// relative position modes (used to move cursor, delete, etc...)

#define	RPM_mMODE			0x000F		// masks out position modes
#define	RPM_CHAR			0x0000
#define	RPM_WORD			0x0001
#define	RPM_LINE			0x0002
#define	RPM_LINEEDGE		0x0003
#define	RPM_PARAGRAPHEDGE	0x0004
#define	RPM_PAGE			0x0005
#define	RPM_DOCEDGE			0x0006
#define	RPM_mBACKWARD		0x8000		// if set, move backward (towards start of file), if clear, move forward

void FixBufferSelections(EDITOR_BUFFER *buffer,UINT32 startOffset,UINT32 oldEndOffset,UINT32 newEndOffset);
void EditorHomeViewToRange(EDITOR_VIEW *view,UINT32 startPosition,UINT32 endPosition,bool useEnd,int horizontalMovement,int verticalMovement);
void EditorHomeViewToSelection(EDITOR_VIEW *view,bool useEnd,int horizontalMovement,int verticalMovement);
void EditorHomeViewToSelectionEdge(EDITOR_VIEW *view,bool useEnd,int horizontalMovement,int verticalMovement);
void EditorVerticalScroll(EDITOR_VIEW *view,INT32 amountToScroll);
void EditorVerticalScrollByPages(EDITOR_VIEW *view,INT32 pagesToScroll);
void EditorHorizontalScroll(EDITOR_VIEW *view,INT32 amountToScroll);
void EditorHorizontalScrollByPages(EDITOR_VIEW *view,INT32 pagesToScroll);
void EditorStartSelectionChange(EDITOR_BUFFER *buffer);
void EditorEndSelectionChange(EDITOR_BUFFER *buffer);
void EditorStartTextChange(EDITOR_BUFFER *buffer);
void EditorEndTextChange(EDITOR_BUFFER *buffer);
bool ReplaceEditorChunks(EDITOR_BUFFER *buffer,UINT32 startOffset,UINT32 endOffset,CHUNK_HEADER *textChunk,UINT32 textOffset,UINT32 numBytes,bool allowUndoMerge);
bool ReplaceEditorText(EDITOR_BUFFER *buffer,UINT32 startOffset,UINT32 endOffset,UINT8 *text,UINT32 numBytes,bool allowUndoMerge);
bool ReplaceEditorFile(EDITOR_BUFFER *buffer,UINT32 startOffset,UINT32 endOffset,const char *path);
void EditorStartReplace(EDITOR_BUFFER *buffer);
void EditorEndReplace(EDITOR_BUFFER *buffer);
bool DeleteAllSelectedText(EDITOR_BUFFER *buffer,SELECTION_UNIVERSE *selectionUniverse);
void EditorGetSelectionInfo(EDITOR_BUFFER *buffer,SELECTION_UNIVERSE *selectionUniverse,UINT32 *startPosition,UINT32 *endPosition,UINT32 *startLine,UINT32 *endLine,UINT32 *startLinePosition,UINT32 *endLinePosition,UINT32 *totalSegments,UINT32 *totalSpan);
UINT8 *EditorNextSelectionToBuffer(EDITOR_BUFFER *buffer,SELECTION_UNIVERSE *selectionUniverse,UINT32 *currentPosition,UINT32 additionalLength,UINT32 *actualLength,bool *atEnd);
bool EditorDeleteSelection(EDITOR_BUFFER *buffer,SELECTION_UNIVERSE *selectionUniverse);
bool EditorSetNormalSelection(EDITOR_BUFFER *buffer,SELECTION_UNIVERSE *selectionUniverse,UINT32 startPosition,UINT32 endPosition);
bool EditorSetColumnarSelection(EDITOR_VIEW *view,UINT32 startLine,UINT32 endLine,INT32 leftX,INT32 rightX,UINT16 trackMode);
bool EditorTrackViewPointer(EDITOR_VIEW *view,INT32 viewXPosition,INT32 viewLine,UINT16 trackMode);
bool EditorMoveCursor(EDITOR_VIEW *view,UINT16 relativeMode);
bool EditorMoveSelection(EDITOR_VIEW *view,UINT16 relativeMode);
bool EditorExpandNormalSelection(EDITOR_VIEW *view,UINT16 relativeMode);
bool EditorReduceNormalSelection(EDITOR_VIEW *view,UINT16 relativeMode);
bool EditorLocateLine(EDITOR_BUFFER *buffer,UINT32 line);
bool EditorSelectAll(EDITOR_BUFFER *buffer);
bool EditorDelete(EDITOR_VIEW *view,UINT16 relativeMode);
bool EditorInsert(EDITOR_BUFFER *buffer,UINT8 *text,UINT32 textLength);
bool EditorAuxInsert(EDITOR_BUFFER *buffer,UINT8 *text,UINT32 textLength);
bool EditorInsertFile(EDITOR_BUFFER *buffer,const char *path);
bool EditorAutoIndent(EDITOR_BUFFER *buffer);
bool ClearBuffer(EDITOR_BUFFER *buffer);
bool LoadBuffer(EDITOR_BUFFER *buffer,const char *path);
bool SaveBuffer(EDITOR_BUFFER *buffer,const char *path,bool setClean);
void LinkViewToBuffer(EDITOR_VIEW *view,EDITOR_BUFFER *buffer);
void UnlinkViewFromBuffer(EDITOR_VIEW *view);
