// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".


// This include file is used to communicate with the GUI
// it contains all prototypes for porting functions that editor
// expects from the GUI implementation.


// ALL the routines that the editor requires from the GUI ------------------------------------------------------------------------------------------

void EditorBeep();
void ReportMessage(const char *format,...);
void GetEditorScreenOffsetAndDimensions(INT32 *xOffset,INT32 *yOffset,UINT32 *width,UINT32 *height);

void GetEditorViewGraphicToTextPosition(EDITOR_VIEW *view,UINT32 linePosition,INT32 xPosition,UINT32 *betweenOffset,UINT32 *charOffset);
void GetEditorViewTextToGraphicPosition(EDITOR_VIEW *view,UINT32 position,INT32 *xPosition,bool limitMax,UINT32 *slopLeft,UINT32 *slopRight);
void GetEditorViewTextInfo(EDITOR_VIEW *view,UINT32 *topLine,UINT32 *numLines,INT32 *leftPixel,UINT32 *numPixels);
void InvalidateViewPortion(EDITOR_VIEW *view,UINT32 startLine,UINT32 endLine,UINT32 startPixel,UINT32 endPixel);
void InvalidateViewSelection(EDITOR_VIEW *view,SELECTION_UNIVERSE *selection);
void InvalidateView(EDITOR_VIEW *view);
void ScrollViewPortion(EDITOR_VIEW *view,UINT32 startLine,UINT32 endLine,INT32 numLines,UINT32 startPixel,UINT32 endPixel,INT32 numPixels);
void SetViewTopLeft(EDITOR_VIEW *view,UINT32 newTopLine,INT32 newLeftPixel);
void SetEditorViewTopLine(EDITOR_VIEW *view,UINT32 lineNumber);
void ResetEditorViewCursorBlink(EDITOR_VIEW *view);
void ViewStyleFontsChanged(EDITOR_VIEW *view);
void ViewEndChange(EDITOR_VIEW *view);
void ViewStartChange(EDITOR_VIEW *view);
void DisposeViewInfo(EDITOR_VIEW *view);
bool CreateViewInfo(EDITOR_VIEW *view,EDITOR_VIEW_DESCRIPTOR *descriptor);

bool EditorColorNameToColor(const char *colorName,EDITOR_COLOR *color);

void UpdateEditorWindows();

void MinimizeDocumentWindow(EDITOR_WINDOW *window);
void UnminimizeDocumentWindow(EDITOR_WINDOW *window);
void SetTopDocumentWindow(EDITOR_WINDOW *window);
EDITOR_WINDOW *GetActiveDocumentWindow();
EDITOR_WINDOW *GetTopDocumentWindow();
bool GetSortedDocumentWindowList(UINT32 *numElements,EDITOR_WINDOW ***list);
EDITOR_VIEW *GetDocumentWindowCurrentView(EDITOR_WINDOW *window);

bool GetDocumentWindowTitle(EDITOR_WINDOW *window,char *title,UINT32 stringBytes);
bool SetDocumentWindowTitle(EDITOR_WINDOW *window,const char *title);
bool GetEditorDocumentWindowForegroundColor(EDITOR_WINDOW *window,EDITOR_COLOR *foregroundColor);
bool SetEditorDocumentWindowForegroundColor(EDITOR_WINDOW *window,EDITOR_COLOR foregroundColor);
bool GetEditorDocumentWindowBackgroundColor(EDITOR_WINDOW *window,EDITOR_COLOR *backgroundColor);
bool SetEditorDocumentWindowBackgroundColor(EDITOR_WINDOW *window,EDITOR_COLOR backgroundColor);
bool GetEditorDocumentWindowRect(EDITOR_WINDOW *window,EDITOR_RECT *rect);
bool SetEditorDocumentWindowRect(EDITOR_WINDOW *window,EDITOR_RECT *rect);
EDITOR_WINDOW *OpenDocumentWindow(EDITOR_BUFFER *buffer,EDITOR_RECT *rect,char *title,char *fontName,UINT32 tabSize,EDITOR_COLOR foreground,EDITOR_COLOR background);
void CloseDocumentWindow(EDITOR_WINDOW *window);

char *CreateWindowTitleFromPath(const char *absolutePath);
char *CreateAbsolutePath(const char *relativePath);
bool LocateStartupScript(char *scriptPath);

EDITOR_MENU *GetRelatedEditorMenu(EDITOR_MENU *menu,int menuRelationship);
void DeactivateEditorMenu(EDITOR_MENU *menu);
void ActivateEditorMenu(EDITOR_MENU *menu);
char *GetEditorMenuName(EDITOR_MENU *menu);
bool SetEditorMenuName(EDITOR_MENU *menu,const char *name);
char *GetEditorMenuAttributes(EDITOR_MENU *menu);
bool SetEditorMenuAttributes(EDITOR_MENU *menu,const char *attributes);
char *GetEditorMenuDataText(EDITOR_MENU *menu);
bool SetEditorMenuDataText(EDITOR_MENU *menu,const char *dataText);
bool GetEditorMenu(int argc,const char *argv[],EDITOR_MENU **menu);
void DisposeEditorMenu(EDITOR_MENU *menu);
EDITOR_MENU *CreateEditorMenu(int argc,const char *argv[],int menuRelationship,const char *name,const char *attributes,const char *dataText,bool active);

bool EditorKeyNameToKeyCode(const char *name,UINT32 *code);
char *EditorKeyCodeToKeyName(UINT32 code);
bool EditorGetKeyPress(UINT32 *keyCode,UINT32 *editorModifiers,bool wait,bool clearBuffered);

void EditorQuit();
void EditorEventLoop(int argc,char *argv[]);
void ClearAbort();
bool CheckAbort();

void ShowBusy();
void ShowNotBusy();

bool ImportClipboard();
bool ExportClipboard();

void GetFontName(EDITOR_FONT *font,char *nameBuffer,UINT32 bufferLength);
void FreeFont(EDITOR_FONT *font);
EDITOR_FONT *LoadFont(const char *name);

EDITOR_FILE *OpenEditorReadFile(const char *path);
EDITOR_FILE *OpenEditorWriteFile(const char *path);
bool ReadEditorFile(EDITOR_FILE *file,UINT8 *buffer,UINT32 bytesToRead,UINT32 *bytesRead);
bool WriteEditorFile(EDITOR_FILE *file,UINT8 *buffer,UINT32 bytesToWrite,UINT32 *bytesWritten);
void CloseEditorFile(EDITOR_FILE *file);

bool OkDialog(const char *text);
bool OkCancelDialog(const char *text,bool *cancel);
bool YesNoCancelDialog(const char *text,bool *yes,bool *cancel);
bool GetSimpleTextDialog(const char *title,char *enteredText,UINT32 stringBytes,bool *cancel);
bool SearchReplaceDialog(EDITOR_BUFFER *searchBuffer,EDITOR_BUFFER *replaceBuffer,bool *backwards,bool *wrapAround,bool *selectionExpr,bool *ignoreCase,bool *limitScope,bool *replaceProc,UINT16 *searchType,bool *cancel);
bool SimpleListBoxDialog(const char *title,UINT32 numElements,const char **listElements,bool *selectedElements,bool *cancel);
bool OpenFileDialog(const char *title,char *fullPath,UINT32 stringBytes,char ***listElements,bool *cancel);
void FreeOpenFileDialogPaths(char **paths);
bool SaveFileDialog(const char *title,char *fullPath,UINT32 stringBytes,bool *cancel);
bool ChoosePathDialog(const char *title,char *fullPath,UINT32 stringBytes,bool *cancel);
bool ChooseFontDialog(const char *title,char *font,UINT32 stringBytes,bool *cancel);

bool WriteBufferTaskData(EDITOR_BUFFER *buffer,UINT8 *data,UINT32 numBytes);
bool SendEOFToBufferTask(EDITOR_BUFFER *buffer);
void DisconnectBufferTask(EDITOR_BUFFER *buffer);
bool UpdateBufferTask(EDITOR_BUFFER *buffer,bool *didUpdate);
bool KillBufferTask(EDITOR_BUFFER *buffer);

void *MNewPtr(UINT32 size);
void *MNewPtrClr(UINT32 size);
void MDisposePtr(void *pointer);
void MMoveMem(void *source,void *dest,UINT32 numBytes);
UINT32 MGetNumAllocatedPointers();

const char *GetEditorLocalVersion();
bool AddSupplementalShellCommands(Tcl_Interp *interpreter);
bool InitEnvironment();
void UnInitEnvironment();
bool EarlyInit();
void EarlyUnInit();

