// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".


typedef struct sparseArrayHeader
{
	UINT32
		offset;					// offset to beginning of element from end of previous element
	UINT32
		length;					// number of elements taken by this entry
} SPARSE_ARRAY_HEADER;

typedef struct sparseArray
{
	ARRAY_UNIVERSE
		elementChunks;			// array of elements
	UINT32
		elementSize;			// size of an element in the array
	UINT32
		totalElements;			// keeps track of the total number of elements in the array (it is an index to one past the last occupied element)
	ARRAY_CHUNK_HEADER
		*cacheChunkHeader;		// stuff used for caching
	UINT32
		cacheOffset;
	UINT32
		cacheCurrentElement;	// tells the number of elements used to this point
} SPARSE_ARRAY;

void DumpSATable(SPARSE_ARRAY *sparseArray);
void DeleteSARange(SPARSE_ARRAY *sparseArray,UINT32 startPosition,UINT32 numElements);
bool InsertSARange(SPARSE_ARRAY *sparseArray,UINT32 startPosition,UINT32 numElements);
bool GetSARecordForPosition(SPARSE_ARRAY *sparseArray,UINT32 position,UINT32 *startPosition,UINT32 *numElements,void **record);
bool GetSARecordAtOrAfterPosition(SPARSE_ARRAY *sparseArray,UINT32 position,UINT32 *startPosition,UINT32 *numElements,void **record);
bool GetSARecordAtOrBeforePosition(SPARSE_ARRAY *sparseArray,UINT32 position,UINT32 *startPosition,UINT32 *numElements,void **record);
bool ClearSARange(SPARSE_ARRAY *sparseArray,UINT32 startPosition,UINT32 numElements);
bool SetSARange(SPARSE_ARRAY *sparseArray,UINT32 startPosition,UINT32 numElements,void *value);
bool IsSAEmpty(SPARSE_ARRAY *sparseArray);
bool GetSAElementEnds(SPARSE_ARRAY *sparseArray,UINT32 *startElementOffset,UINT32 *endElementOffset);
void UnInitSA(SPARSE_ARRAY *sparseArray);
bool InitSA(SPARSE_ARRAY *sparseArray,UINT32 elementSize);
