// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".

typedef struct dialogItem	// item in list of items for given dialog (built during dialog creation)
{
	struct dialog
		*dialog;			// pointer to the dialog this item is in
	void
		(*disposeProc)(struct dialogItem *item);	// routine to call to get rid of item's local storage
	void
		(*drawProc)(struct dialogItem *item,Pixmap imageBuffer,Region updateRegion);	// routine to call when drawing item
	bool
		(*trackProc)(struct dialogItem *item,XEvent *event);		// routine to call when attempting to track the item (returns false if initial point not in item)
	void
		(*resizeProc)(struct dialogItem *item);		// call this after the window has changed size to resize/reposition the item
	bool
		(*earlyKeyProc)(struct dialogItem *item,XEvent *event);		// if this item wants certain control keys before the default item can get them, it defines this
	bool
		wantFocus;									// tells the dialog manager that this item desires to be in the focus chain
	void
		(*focusChangeProc)(struct dialogItem *item);	// called by dialog manager to tell item that focus is changing
	void
		(*focusKeyProc)(struct dialogItem *item,XEvent *event);	// when keys come in for this focussed item, this is called
	void
		(*focusPeriodicProc)(struct dialogItem *item);	// if this item wants to flash the cursor or whatever, it can set up this routine
	void
		*itemStruct;		// pointer to any data this item requires (used by create routines if needed)
	struct dialogItem
		*previousItem,
		*nextItem;
} DIALOG_ITEM;

typedef struct dialog		// list of dialog items
{
	EDITOR_WINDOW
		*parentWindow;		// the editor window that this dialog is in
	EDITOR_COLOR
		backgroundColor;	// color to use when drawing background of the dialog
	bool
		dialogComplete;		// set to true by dialog procs to tell when dialog is finished, and should be closed
	DIALOG_ITEM
		*firstItem,			// pointer to first item in the dialog (or NULL)
		*lastItem;			// pointer to last item in the dialog (or NULL)
	DIALOG_ITEM
		*focusItem;			// pointer to item with focus (or NULL)
	void
		*dialogLocal;		// place for local dialog routines to hang any data they need
} DIALOG;


DIALOG_ITEM *NewDialogItem(DIALOG *dialog,bool (*createProc)(DIALOG_ITEM *item,void *itemDescriptor),void *itemDescriptor);
void DisposeDialogItem(DIALOG_ITEM *dialogItem);
void UpdateDialogWindow(EDITOR_WINDOW *window);
void AdjustDialogWindowForNewSize(EDITOR_WINDOW *window);
void DialogPeriodicProc(EDITOR_WINDOW *window);
void DialogTakeFocus(DIALOG_ITEM *item);
void DialogWindowEvent(EDITOR_WINDOW *window,XEvent *event);
bool OkDialog(const char *text);
bool OkCancelDialog(const char *text,bool *cancel);
bool YesNoCancelDialog(const char *text,bool *yes,bool *cancel);
bool GetSimpleTextDialog(const char *title,char *enteredText,UINT32 stringBytes,bool *cancel);
bool SearchReplaceDialog(EDITOR_BUFFER *searchBuffer,EDITOR_BUFFER *replaceBuffer,bool *backwards,bool *wrapAround,bool *selectionExpr,bool *ignoreCase,bool *limitScope,bool *replaceProc,UINT16 *searchType,bool *cancel);
bool SimpleListBoxDialog(const char *title,UINT32 numElements,const char **listElements,bool *selectedElements,bool *cancel);
bool OpenFileDialog(const char *title,char *fullPath,UINT32 stringBytes,char ***listElements,bool *cancel);
void FreeOpenFileDialogPaths(char **paths);
bool SaveFileDialog(const char *title,char *fullPath,UINT32 stringBytes,bool *cancel);
bool ChoosePathDialog(const char *title,char *fullPath,UINT32 stringBytes,bool *cancel);
bool ChooseFontDialog(const char *title,char *font,UINT32 stringBytes,bool *cancel);
