// Scroll bar handling
// Copyright (C) 2000 Core Technologies.

// This file is part of e93.
//
// e93 is free software; you can redistribute it and/or modify
// it under the terms of the e93 LICENSE AGREEMENT.
//
// e93 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// e93 LICENSE AGREEMENT for more details.
//
// You should have received a copy of the e93 LICENSE AGREEMENT
// along with e93; see the file "LICENSE.TXT".

#include	"includes.h"

// Some terminology:
// low arrow is the arrow that attempts to move the thumb to lower values
// high arrow moves it higher

#define	SCROLL_REPEAT_DELAY		20				// number of ticks to wait before repeating

static void GetScrollBarControlRects(SCROLL_BAR *scrollBar,EDITOR_RECT *lowArrowRect,EDITOR_RECT *highArrowRect,EDITOR_RECT *thumbRect,EDITOR_RECT *slideRect,EDITOR_RECT *pageLowRect,EDITOR_RECT *pageHighRect)
// return the rectangles that enclose the various pieces of scrollBar
// if the scroll bar starts getting too small, certain rects may be returned with 0 width or
// height
{
	UINT32
		lowLength,
		highLength,
		thumbLength,
		thumbSpace;

	if(scrollBar->length<2*ARROW_LENGTH)			// problem fitting arrows?
	{
		lowLength=(scrollBar->length+1)/2;			// get the length of the two arrows, and work backwards from there
		highLength=scrollBar->length-lowLength;
	}
	else
	{
		lowLength=highLength=ARROW_LENGTH;
	}
	if(scrollBar->horizontal)
	{
		lowArrowRect->x=scrollBar->x;
		lowArrowRect->w=lowLength;
		lowArrowRect->y=scrollBar->y;
		lowArrowRect->h=SCROLL_BAR_WIDTH;

		highArrowRect->w=highLength;
		highArrowRect->x=(scrollBar->x+scrollBar->length)-highLength;
		highArrowRect->y=scrollBar->y;
		highArrowRect->h=SCROLL_BAR_WIDTH;

		slideRect->x=lowArrowRect->x+lowArrowRect->w;
		slideRect->w=highArrowRect->x-slideRect->x;
		slideRect->y=scrollBar->y;
		slideRect->h=SCROLL_BAR_WIDTH;

		thumbLength=slideRect->w/(scrollBar->thumbMax+1);

		if(thumbLength<THUMB_LENGTH)
		{
			thumbLength=THUMB_LENGTH;
		}

		if(thumbLength>=slideRect->w)
		{
			thumbRect->w=slideRect->w;
		}
		else
		{
			thumbRect->w=thumbLength;
		}
		thumbSpace=slideRect->w-thumbRect->w;
		if(scrollBar->thumbMax)
		{
			thumbRect->x=slideRect->x+(thumbSpace*scrollBar->thumbPosition)/scrollBar->thumbMax;
		}
		else
		{
			thumbRect->x=slideRect->x;
		}
		thumbRect->y=scrollBar->y;
		thumbRect->h=SCROLL_BAR_WIDTH;

		pageLowRect->x=slideRect->x;
		pageLowRect->y=slideRect->y;
		pageLowRect->w=thumbRect->x-slideRect->x;
		pageLowRect->h=slideRect->h;

		pageHighRect->x=thumbRect->x+thumbRect->w;
		pageHighRect->y=slideRect->y;
		pageHighRect->w=slideRect->x+slideRect->w-pageHighRect->x;
		pageHighRect->h=slideRect->h;
	}
	else
	{
		lowArrowRect->x=scrollBar->x;
		lowArrowRect->w=SCROLL_BAR_WIDTH;
		lowArrowRect->y=scrollBar->y;
		lowArrowRect->h=lowLength;

		highArrowRect->x=scrollBar->x;
		highArrowRect->w=SCROLL_BAR_WIDTH;
		highArrowRect->y=(scrollBar->y+scrollBar->length)-highLength;
		highArrowRect->h=highLength;

		slideRect->x=scrollBar->x;
		slideRect->w=SCROLL_BAR_WIDTH;
		slideRect->y=lowArrowRect->y+lowArrowRect->h;
		slideRect->h=highArrowRect->y-slideRect->y;

		thumbRect->x=scrollBar->x;
		thumbRect->w=SCROLL_BAR_WIDTH;

		thumbLength=slideRect->h/(scrollBar->thumbMax+1);

		if(thumbLength<THUMB_LENGTH)
		{
			thumbLength=THUMB_LENGTH;
		}

		if(thumbLength>=slideRect->h)
		{
			thumbRect->h=slideRect->h;
		}
		else
		{
			thumbRect->h=thumbLength;
		}
		thumbSpace=slideRect->h-thumbRect->h;

		if(scrollBar->thumbMax)
		{
			thumbRect->y=slideRect->y+(thumbSpace*scrollBar->thumbPosition)/scrollBar->thumbMax;
		}
		else
		{
			thumbRect->y=slideRect->y;
		}

		pageLowRect->x=slideRect->x;
		pageLowRect->y=slideRect->y;
		pageLowRect->w=slideRect->w;
		pageLowRect->h=thumbRect->y-slideRect->y;

		pageHighRect->x=slideRect->x;
		pageHighRect->y=thumbRect->y+thumbRect->h;
		pageHighRect->w=slideRect->w;
		pageHighRect->h=slideRect->y+slideRect->h-pageHighRect->y;
	}
}

bool PointInScrollBar(SCROLL_BAR *scrollBar,INT32 x,INT32 y)
// return true if parent window relative x/y are in scrollBar
// false if not
{
	EDITOR_RECT
		rect;

	rect.x=scrollBar->x;
	rect.y=scrollBar->y;
	if(scrollBar->horizontal)
	{
		rect.w=scrollBar->length;
		rect.h=SCROLL_BAR_WIDTH;
	}
	else
	{
		rect.w=SCROLL_BAR_WIDTH;
		rect.h=scrollBar->length;
	}
	return(PointInRECT(x,y,&rect));
}

static void InvalidateScrollBar(SCROLL_BAR *scrollBar)
// invalidate the scroll bar within its parent window
{
	EDITOR_RECT
		invalidRect;

	invalidRect.x=scrollBar->x;
	invalidRect.y=scrollBar->y;
	if(scrollBar->horizontal)
	{
		invalidRect.w=scrollBar->length;
		invalidRect.h=SCROLL_BAR_WIDTH;
	}
	else
	{
		invalidRect.w=SCROLL_BAR_WIDTH;
		invalidRect.h=scrollBar->length;
	}
	InvalidateWindowRect(scrollBar->parentWindow,&invalidRect);
}

static void DrawLeftArrow(Window imageBuffer,EDITOR_RECT *rect,bool highlight)
// draw a left arrow control into rect of window
{
	if(rect->w&&rect->h)
	{
		if(!highlight)
		{
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(GRAY_3));
		}
		else
		{
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(WHITE));
		}
		XSetBackground(xDisplay,xGraphicsContext,EditorColorToXPixel(GRAY_0));
		XCopyPlane(xDisplay,leftArrowPixmap,imageBuffer,xGraphicsContext,0,ARROW_LENGTH-rect->h,rect->w,rect->h,rect->x,rect->y,1);
		XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(BLACK));
		XDrawRectangle(xDisplay,imageBuffer,xGraphicsContext,rect->x,rect->y,rect->w-1,rect->h-1);
	}
}

static void DrawRightArrow(Window imageBuffer,EDITOR_RECT *rect,bool highlight)
// draw a right arrow control into rect of window
{
	if(rect->w&&rect->h)
	{
		if(!highlight)
		{
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(GRAY_3));
		}
		else
		{
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(WHITE));
		}
		XSetBackground(xDisplay,xGraphicsContext,EditorColorToXPixel(GRAY_0));
		XCopyPlane(xDisplay,rightArrowPixmap,imageBuffer,xGraphicsContext,0,ARROW_LENGTH-rect->h,rect->w,rect->h,rect->x,rect->y,1);
		XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(BLACK));
		XDrawRectangle(xDisplay,imageBuffer,xGraphicsContext,rect->x,rect->y,rect->w-1,rect->h-1);
	}
}

static void DrawUpArrow(Window imageBuffer,EDITOR_RECT *rect,bool highlight)
// draw an up arrow control into rect of window
{
	if(rect->w&&rect->h)
	{
		if(!highlight)
		{
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(GRAY_3));
		}
		else
		{
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(WHITE));
		}
		XSetBackground(xDisplay,xGraphicsContext,EditorColorToXPixel(GRAY_0));
		XCopyPlane(xDisplay,upArrowPixmap,imageBuffer,xGraphicsContext,0,ARROW_LENGTH-rect->h,rect->w,rect->h,rect->x,rect->y,1);
		XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(BLACK));
		XDrawRectangle(xDisplay,imageBuffer,xGraphicsContext,rect->x,rect->y,rect->w-1,rect->h-1);
	}
}

static void DrawDownArrow(Window imageBuffer,EDITOR_RECT *rect,bool highlight)
// draw a down arrow control into rect of window
{
	if(rect->w&&rect->h)
	{
		if(!highlight)
		{
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(GRAY_3));
		}
		else
		{
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(WHITE));
		}
		XSetBackground(xDisplay,xGraphicsContext,EditorColorToXPixel(GRAY_0));
		XCopyPlane(xDisplay,downArrowPixmap,imageBuffer,xGraphicsContext,0,ARROW_LENGTH-rect->h,rect->w,rect->h,rect->x,rect->y,1);
		XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(BLACK));
		XDrawRectangle(xDisplay,imageBuffer,xGraphicsContext,rect->x,rect->y,rect->w-1,rect->h-1);
	}
}

static void DrawPageIndicator(Window imageBuffer,EDITOR_RECT *rect,bool horizontal,bool highlight)
// draw a page control into rect of window
{
	if(rect->w&&rect->h)
	{
		if(!highlight)
		{
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(0x868693));
		}
		else
		{
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(GRAY_1));
		}
		if(horizontal)
		{
			if(rect->h>1)
			{
				XFillRectangle(xDisplay,imageBuffer,xGraphicsContext,rect->x,rect->y+1,rect->w,rect->h-2);	// fill the inset rectangle
			}
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(BLACK));
			XDrawLine(xDisplay,imageBuffer,xGraphicsContext,rect->x,rect->y,rect->x+rect->w-1,rect->y);
			XDrawLine(xDisplay,imageBuffer,xGraphicsContext,rect->x,rect->y+rect->h-1,rect->x+rect->w-1,rect->y+rect->h-1);
		}
		else
		{
			if(rect->w>1)
			{
				XFillRectangle(xDisplay,imageBuffer,xGraphicsContext,rect->x+1,rect->y,rect->w-2,rect->h);	// fill the inset rectangle
			}
			XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(BLACK));
			XDrawLine(xDisplay,imageBuffer,xGraphicsContext,rect->x,rect->y,rect->x,rect->y+rect->h-1);
			XDrawLine(xDisplay,imageBuffer,xGraphicsContext,rect->x+rect->w-1,rect->y,rect->x+rect->w-1,rect->y+rect->h-1);
		}
	}
}

static void DrawThumb(Window imageBuffer,EDITOR_RECT *rect,bool highlight)
// draw a thumb control into rect of window
{
	if(rect->w&&rect->h)
	{
		if(rect->w>1&&rect->h>1)
		{
			if(!highlight)
			{
				XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(GRAY_3));
			}
			else
			{
				XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(WHITE));
			}
			XFillRectangle(xDisplay,imageBuffer,xGraphicsContext,rect->x+1,rect->y+1,rect->w-2,rect->h-2);	// fill the inset rectangle
		}
		XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(BLACK));
		XDrawRectangle(xDisplay,imageBuffer,xGraphicsContext,rect->x,rect->y,rect->w-1,rect->h-1);			// draw the outline
		XSetForeground(xDisplay,xGraphicsContext,EditorColorToXPixel(WHITE));
		XDrawLine(xDisplay,imageBuffer,xGraphicsContext,rect->x+1,rect->y+1,rect->x+rect->w-2,rect->y+1);
		XDrawLine(xDisplay,imageBuffer,xGraphicsContext,rect->x+1,rect->y+1,rect->x+1,rect->y+rect->h-2);
	}
}

void DrawScrollBar(SCROLL_BAR *scrollBar,Pixmap imageBuffer,Region updateRegion)
// redraw scrollBar in its parent window, obey the invalid area of the parent's window
{
	EDITOR_RECT
		lowArrowRect,
		highArrowRect,
		thumbRect,
		slideRect,
		pageLowRect,
		pageHighRect;

	XSetRegion(xDisplay,xGraphicsContext,updateRegion);								// set clipping to what's invalid
	GetScrollBarControlRects(scrollBar,&lowArrowRect,&highArrowRect,&thumbRect,&slideRect,&pageLowRect,&pageHighRect);
	if(scrollBar->horizontal)
	{
		DrawLeftArrow(imageBuffer,&lowArrowRect,scrollBar->highlightLowArrow);
		DrawRightArrow(imageBuffer,&highArrowRect,scrollBar->highlightHighArrow);
	}
	else
	{
		DrawUpArrow(imageBuffer,&lowArrowRect,scrollBar->highlightLowArrow);
		DrawDownArrow(imageBuffer,&highArrowRect,scrollBar->highlightHighArrow);
	}
	DrawPageIndicator(imageBuffer,&pageLowRect,scrollBar->horizontal,scrollBar->highlightLowPage);
	DrawPageIndicator(imageBuffer,&pageHighRect,scrollBar->horizontal,scrollBar->highlightHighPage);
	DrawThumb(imageBuffer,&thumbRect,scrollBar->highlightThumb);
	XSetClipMask(xDisplay,xGraphicsContext,None);									// get rid of clip mask
}

static bool TrackLowArrow(SCROLL_BAR *scrollBar,XEvent *event)
// track the low arrow of scrollBar
{
	EDITOR_RECT
		lowArrowRect,
		highArrowRect,
		thumbRect,
		slideRect,
		pageLowRect,
		pageHighRect;
	UINT32
		initialTime;

	GetScrollBarControlRects(scrollBar,&lowArrowRect,&highArrowRect,&thumbRect,&slideRect,&pageLowRect,&pageHighRect);
	if((event->xbutton.button==Button1)&&PointInRECT(event->xbutton.x,event->xbutton.y,&lowArrowRect))
	{
		scrollBar->highlightLowArrow=true;
		InvalidateScrollBar(scrollBar);
		UpdateEditorWindows();												// redraw anything that is needed
		if(scrollBar->stepLowerProc)
		{
			scrollBar->stepLowerProc(scrollBar,false,scrollBar->procParameters);
		}
		initialTime=0;
		while(StillDown(Button1,1))
		{
			if(initialTime>SCROLL_REPEAT_DELAY)
			{
				if(scrollBar->stepLowerProc)
				{
					scrollBar->stepLowerProc(scrollBar,true,scrollBar->procParameters);
				}
			}
			else
			{
				initialTime++;
			}
		}
		scrollBar->highlightLowArrow=false;
		InvalidateScrollBar(scrollBar);
		return(true);
	}
	return(false);
}

static bool TrackHighArrow(SCROLL_BAR *scrollBar,XEvent *event)
// track the high arrow of scrollBar
{
	EDITOR_RECT
		lowArrowRect,
		highArrowRect,
		thumbRect,
		slideRect,
		pageLowRect,
		pageHighRect;
	UINT32
		initialTime;

	GetScrollBarControlRects(scrollBar,&lowArrowRect,&highArrowRect,&thumbRect,&slideRect,&pageLowRect,&pageHighRect);
	if((event->xbutton.button==Button1)&&PointInRECT(event->xbutton.x,event->xbutton.y,&highArrowRect))
	{
		scrollBar->highlightHighArrow=true;
		InvalidateScrollBar(scrollBar);
		UpdateEditorWindows();												// redraw anything that is needed
		if(scrollBar->stepHigherProc)
		{
			scrollBar->stepHigherProc(scrollBar,false,scrollBar->procParameters);
		}
		initialTime=0;
		while(StillDown(Button1,1))
		{
			if(initialTime>SCROLL_REPEAT_DELAY)
			{
				if(scrollBar->stepHigherProc)
				{
					scrollBar->stepHigherProc(scrollBar,true,scrollBar->procParameters);
				}
			}
			else
			{
				initialTime++;
			}
		}
		scrollBar->highlightHighArrow=false;
		InvalidateScrollBar(scrollBar);
		return(true);
	}
	return(false);
}

static bool TrackLowPage(SCROLL_BAR *scrollBar,XEvent *event)
// track the low page of scrollBar
{
	EDITOR_RECT
		lowArrowRect,
		highArrowRect,
		thumbRect,
		slideRect,
		pageLowRect,
		pageHighRect;
	UINT32
		initialTime;

	GetScrollBarControlRects(scrollBar,&lowArrowRect,&highArrowRect,&thumbRect,&slideRect,&pageLowRect,&pageHighRect);
	if((event->xbutton.button==Button1)&&PointInRECT(event->xbutton.x,event->xbutton.y,&pageLowRect))
	{
		scrollBar->highlightLowPage=true;
		InvalidateScrollBar(scrollBar);
		UpdateEditorWindows();												// redraw anything that is needed
		if(scrollBar->pageLowerProc)
		{
			scrollBar->pageLowerProc(scrollBar,false,scrollBar->procParameters);
		}
		initialTime=0;
		while(StillDown(Button1,1))
		{
			if(initialTime>SCROLL_REPEAT_DELAY)
			{
				if(scrollBar->pageLowerProc)
				{
					scrollBar->pageLowerProc(scrollBar,true,scrollBar->procParameters);
				}
			}
			else
			{
				initialTime++;
			}
		}
		scrollBar->highlightLowPage=false;
		InvalidateScrollBar(scrollBar);
		return(true);
	}
	return(false);
}

static bool TrackHighPage(SCROLL_BAR *scrollBar,XEvent *event)
// track the high page of scrollBar
{
	EDITOR_RECT
		lowArrowRect,
		highArrowRect,
		thumbRect,
		slideRect,
		pageLowRect,
		pageHighRect;
	UINT32
		initialTime;

	GetScrollBarControlRects(scrollBar,&lowArrowRect,&highArrowRect,&thumbRect,&slideRect,&pageLowRect,&pageHighRect);
	if((event->xbutton.button==Button1)&&PointInRECT(event->xbutton.x,event->xbutton.y,&pageHighRect))
	{
		scrollBar->highlightHighPage=true;
		InvalidateScrollBar(scrollBar);
		UpdateEditorWindows();												// redraw anything that is needed
		if(scrollBar->pageHigherProc)
		{
			scrollBar->pageHigherProc(scrollBar,false,scrollBar->procParameters);
		}
		initialTime=0;
		while(StillDown(Button1,1))
		{
			if(initialTime>SCROLL_REPEAT_DELAY)
			{
				if(scrollBar->pageHigherProc)
				{
					scrollBar->pageHigherProc(scrollBar,true,scrollBar->procParameters);
				}
			}
			else
			{
				initialTime++;
			}
		}
		scrollBar->highlightHighPage=false;
		InvalidateScrollBar(scrollBar);
		return(true);
	}
	return(false);
}

static UINT32 CalculateThumbValue(UINT32 currentPixel,UINT32 numPixels,UINT32 thumbMax)
// given a pixel position within a range of pixels, return the thumbPosition within
// thumbMax
{
	if(numPixels)
	{
		return(((currentPixel*thumbMax)+(numPixels/2))/numPixels);
	}
	else
	{
		return(0);
	}
}

static bool TrackMetaThumb(SCROLL_BAR *scrollBar,XEvent *event)
// see if a button other that Button1 is down in the scroll bar, if so
// track the thumb of scrollBar
// while tracking the thumb of scrollBar, it is rude for the application to change
// the position of the bar on the screen,
// if this is done, the results will be unpredictable
{
	EDITOR_RECT
		lowArrowRect,
		highArrowRect,
		thumbRect,
		slideRect,
		pageLowRect,
		pageHighRect;
	Window
		root,
		child;
	int
		rootX,
		rootY,
		windowX,
		windowY,
		testIndex;
	unsigned int
		state;
	UINT32
		scrollablePixels,
		lastValue,
		newValue;

	GetScrollBarControlRects(scrollBar,&lowArrowRect,&highArrowRect,&thumbRect,&slideRect,&pageLowRect,&pageHighRect);
	if((event->xbutton.button!=Button1)&&PointInRECT(event->xbutton.x,event->xbutton.y,&slideRect))
	{
		scrollBar->highlightThumb=true;
		InvalidateScrollBar(scrollBar);
		UpdateEditorWindows();									// redraw anything that is needed
		if(scrollBar->horizontal)
		{
			scrollablePixels=slideRect.w-thumbRect.w;
			lastValue=CalculateThumbValue(thumbRect.x-slideRect.x,scrollablePixels,scrollBar->thumbMax);
			while(StillDown(event->xbutton.button,1))
			{
				if(XQueryPointer(xDisplay,((WINDOW_LIST_ELEMENT *)scrollBar->parentWindow->userData)->xWindow,&root,&child,&rootX,&rootY,&windowX,&windowY,&state))
				{
					testIndex=windowX-slideRect.x;				// get current pointer position
					testIndex-=thumbRect.w/2;					// add to initial offset
					if(testIndex<0)								// pin to range
					{
						testIndex=0;
					}
					if(testIndex>(int)scrollablePixels)
					{
						testIndex=scrollablePixels;
					}
					newValue=CalculateThumbValue((UINT32)testIndex,scrollablePixels,scrollBar->thumbMax);
					if(newValue!=lastValue)
					{
						if(scrollBar->thumbProc)
						{
							scrollBar->thumbProc(scrollBar,newValue,scrollBar->procParameters);
						}
						lastValue=newValue;
					}
				}
			}
		}
		else
		{
			scrollablePixels=slideRect.h-thumbRect.h;
			lastValue=CalculateThumbValue(thumbRect.y-slideRect.y,scrollablePixels,scrollBar->thumbMax);
			while(StillDown(event->xbutton.button,1))
			{
				if(XQueryPointer(xDisplay,((WINDOW_LIST_ELEMENT *)scrollBar->parentWindow->userData)->xWindow,&root,&child,&rootX,&rootY,&windowX,&windowY,&state))
				{
					testIndex=windowY-slideRect.y;		// get current pointer position
					testIndex-=thumbRect.h/2;			// add to initial offset
					if(testIndex<0)						// pin to range
					{
						testIndex=0;
					}
					if(testIndex>(int)scrollablePixels)
					{
						testIndex=scrollablePixels;
					}
					newValue=CalculateThumbValue((UINT32)testIndex,scrollablePixels,scrollBar->thumbMax);
					if(newValue!=lastValue)
					{
						if(scrollBar->thumbProc)
						{
							scrollBar->thumbProc(scrollBar,newValue,scrollBar->procParameters);
						}
						lastValue=newValue;
					}
				}
			}
		}
		scrollBar->highlightThumb=false;
		InvalidateScrollBar(scrollBar);
		return(true);
	}
	return(false);
}

static bool TrackThumb(SCROLL_BAR *scrollBar,XEvent *event)
// track the thumb of scrollBar
// while tracking the thumb of scrollBar, it is rude for the application to change
// the position of the bar on the screen,
// if this is done, the results will be unpredictable
{
	EDITOR_RECT
		lowArrowRect,
		highArrowRect,
		thumbRect,
		slideRect,
		pageLowRect,
		pageHighRect;
	Window
		root,
		child;
	int
		rootX,
		rootY,
		windowX,
		windowY,
		testIndex;
	unsigned int
		state;
	UINT32
		scrollablePixels,
		initialOffset,
		lastValue,
		newValue;

	GetScrollBarControlRects(scrollBar,&lowArrowRect,&highArrowRect,&thumbRect,&slideRect,&pageLowRect,&pageHighRect);
	if((event->xbutton.button==Button1)&&PointInRECT(event->xbutton.x,event->xbutton.y,&thumbRect))
	{
		scrollBar->highlightThumb=true;
		InvalidateScrollBar(scrollBar);
		UpdateEditorWindows();									// redraw anything that is needed
		if(scrollBar->horizontal)
		{
			scrollablePixels=slideRect.w-thumbRect.w;
			initialOffset=thumbRect.x-slideRect.x;
			lastValue=CalculateThumbValue(initialOffset,scrollablePixels,scrollBar->thumbMax);
			while(StillDown(Button1,1))
			{
				if(XQueryPointer(xDisplay,((WINDOW_LIST_ELEMENT *)scrollBar->parentWindow->userData)->xWindow,&root,&child,&rootX,&rootY,&windowX,&windowY,&state))
				{
					testIndex=windowX-event->xbutton.x;			// get current pointer position
					testIndex+=(INT32)initialOffset;			// add to initial offset
					if(testIndex<0)								// pin to range
					{
						testIndex=0;
					}
					if(testIndex>(int)scrollablePixels)
					{
						testIndex=scrollablePixels;
					}
					newValue=CalculateThumbValue((UINT32)testIndex,scrollablePixels,scrollBar->thumbMax);
					if(newValue!=lastValue)
					{
						if(scrollBar->thumbProc)
						{
							scrollBar->thumbProc(scrollBar,newValue,scrollBar->procParameters);
						}
						lastValue=newValue;
					}
				}
			}
		}
		else
		{
			scrollablePixels=slideRect.h-thumbRect.h;
			initialOffset=thumbRect.y-slideRect.y;
			lastValue=CalculateThumbValue(initialOffset,scrollablePixels,scrollBar->thumbMax);
			while(StillDown(Button1,1))
			{
				if(XQueryPointer(xDisplay,((WINDOW_LIST_ELEMENT *)scrollBar->parentWindow->userData)->xWindow,&root,&child,&rootX,&rootY,&windowX,&windowY,&state))
				{
					testIndex=windowY-event->xbutton.y;			// get current pointer position
					testIndex+=(INT32)initialOffset;			// add to initial offset
					if(testIndex<0)								// pin to range
					{
						testIndex=0;
					}
					if(testIndex>(int)scrollablePixels)
					{
						testIndex=scrollablePixels;
					}
					newValue=CalculateThumbValue((UINT32)testIndex,scrollablePixels,scrollBar->thumbMax);
					if(newValue!=lastValue)
					{
						if(scrollBar->thumbProc)
						{
							scrollBar->thumbProc(scrollBar,newValue,scrollBar->procParameters);
						}
						lastValue=newValue;
					}
				}
			}
		}
		scrollBar->highlightThumb=false;
		InvalidateScrollBar(scrollBar);
		return(true);
	}
	return(false);
}

bool TrackScrollBar(SCROLL_BAR *scrollBar,XEvent *event)
// a button was pressed, see if it was in scrollBar, if so, track it, and return true
// if it was not within the scroll bar, return false
{
	if(!TrackLowArrow(scrollBar,event))
	{
		if(!TrackHighArrow(scrollBar,event))
		{
			if(!TrackMetaThumb(scrollBar,event))
			{
				if(!TrackLowPage(scrollBar,event))
				{
					if(!TrackHighPage(scrollBar,event))
					{
						if(!TrackThumb(scrollBar,event))
						{
							return(false);
						}
					}
				}
			}
		}
	}
	return(true);
}

void RepositionScrollBar(SCROLL_BAR *scrollBar,INT32 x,INT32 y,UINT32 length,bool horizontal)
// change the position of scrollBar within its parent
// be nice, and only invalidate it if it actually changes
{
	if((scrollBar->x!=x)||(scrollBar->y!=y)||(scrollBar->length!=length)||(scrollBar->horizontal!=horizontal))
	{
		InvalidateScrollBar(scrollBar);					// invalidate where it is now
		scrollBar->x=x;
		scrollBar->y=y;
		scrollBar->length=length;
		scrollBar->horizontal=horizontal;
		InvalidateScrollBar(scrollBar);					// invalidate where it is going
	}
}

void AdjustScrollBar(SCROLL_BAR *scrollBar,UINT32 thumbMax,UINT32 thumbPosition)
// adjust the thumbMax, and position of scrollBar
// be nice, and only invalidate it if it actually changes
{
	EDITOR_RECT
		lowArrowRect,
		highArrowRect,
		thumbRect,
		slideRect,
		pageLowRect,
		pageHighRect;

	if((scrollBar->thumbMax!=thumbMax)||(scrollBar->thumbPosition!=thumbPosition))
	{
		GetScrollBarControlRects(scrollBar,&lowArrowRect,&highArrowRect,&thumbRect,&slideRect,&pageLowRect,&pageHighRect);	// invalidate where the thumb was
		InvalidateWindowRect(scrollBar->parentWindow,&pageLowRect);
		InvalidateWindowRect(scrollBar->parentWindow,&thumbRect);
		InvalidateWindowRect(scrollBar->parentWindow,&pageHighRect);
		scrollBar->thumbMax=thumbMax;
		scrollBar->thumbPosition=thumbPosition;
		GetScrollBarControlRects(scrollBar,&lowArrowRect,&highArrowRect,&thumbRect,&slideRect,&pageLowRect,&pageHighRect);	// invalidate where the thumb is going
		InvalidateWindowRect(scrollBar->parentWindow,&pageLowRect);
		InvalidateWindowRect(scrollBar->parentWindow,&thumbRect);
		InvalidateWindowRect(scrollBar->parentWindow,&pageHighRect);
	}
}

SCROLL_BAR *CreateScrollBar(EDITOR_WINDOW *window,SCROLL_BAR_DESCRIPTOR *description)
// create a scroll bar in window, invalidate the area of window
// where the scroll bar is to be placed
// return a pointer to it if successful
// SetError, return NULL if not
{
	SCROLL_BAR
		*scrollBar;

	if((scrollBar=(SCROLL_BAR *)MNewPtr(sizeof(SCROLL_BAR))))		// create scroll bar data structure
	{
		scrollBar->parentWindow=window;
		scrollBar->x=description->x;
		scrollBar->y=description->y;
		scrollBar->length=description->length;
		scrollBar->horizontal=description->horizontal;
		scrollBar->thumbMax=description->thumbMax;
		scrollBar->thumbPosition=description->thumbPosition;

		scrollBar->highlightLowArrow=false;
		scrollBar->highlightHighArrow=false;
		scrollBar->highlightLowPage=false;
		scrollBar->highlightHighPage=false;
		scrollBar->highlightThumb=false;

		scrollBar->thumbProc=description->thumbProc;
		scrollBar->stepLowerProc=description->stepLowerProc;
		scrollBar->stepHigherProc=description->stepHigherProc;
		scrollBar->pageLowerProc=description->pageLowerProc;
		scrollBar->pageHigherProc=description->pageHigherProc;
		scrollBar->procParameters=description->procParameters;
		InvalidateScrollBar(scrollBar);
	}
	return(scrollBar);
}

void DisposeScrollBar(SCROLL_BAR *scrollBar)
// unlink scrollBar from its parent window, and delete it
// invalidate the area of the window where the scroll bar was
{
	InvalidateScrollBar(scrollBar);					// invalidate where it is now
	MDisposePtr(scrollBar);
}
